﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Windows.Threading;

	using Ecng.Common;
	using Ecng.Xaml;

	/// <summary>
	/// Синхронизованный графический объект. Оборачивает обычный объект для того, чтобы все события приходили в GUI потоке.
	/// </summary>
	public class GuiObject : Disposable
	{
		/// <summary>
		/// Создать синхронизованный объект. Интервал опроса накопившихся событий для оправки синхронизованным способом равен 1 секунде.
		/// </summary>
		public GuiObject()
			: this(Dispatcher.CurrentDispatcher)
		{
		}

		/// <summary>
		/// Создать синхронизованный объект. Интервал опроса накопившихся событий для оправки синхронизованным способом равен 1 секунде.
		/// </summary>
		/// <param name="dispatcher">Специальный класс для синхронизации с GUI потоком.</param>
		public GuiObject(Dispatcher dispatcher)
			: this(dispatcher, TimeSpan.FromSeconds(1))
		{
		}

		/// <summary>
		/// Создать синхронизованный объект.
		/// </summary>
		/// <param name="dispatcher">Специальный класс для синхронизации с GUI потоком.</param>
		/// <param name="interval">Интервал опроса накопившихся событий для оправки синхронизованным способом.</param>
		public GuiObject(Dispatcher dispatcher, TimeSpan interval)
		{
			this.EventDispatcher = new XamlEventDispatcher(error => { throw error; }, interval, dispatcher);
		}

		/// <summary>
		/// Обработчик очереди событий.
		/// </summary>
		protected XamlEventDispatcher EventDispatcher { get; private set; }

		/// <summary>
		/// Освободить занятые ресурсы.
		/// </summary>
		protected override void DisposeManaged()
		{
			this.EventDispatcher.Dispose();
			base.DisposeManaged();
		}
	}
}