namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.Generic;
	using System.Windows.Threading;

	using Ecng.Common;
	using Ecng.Trading.BusinessEntities;

	/// <summary>
	///  .   <see cref="ITrader"/>    ,      GUI .
	/// </summary>
	/// <typeparam name="T"> ,   .</typeparam>
	public class GuiTrader<T> : GuiObject, ITrader
		where T : ITrader
	{
		/// <summary>
		///   .          1 .
		/// </summary>
		/// <param name="trader">,     <see cref="GuiTrader{T}"/>.</param>
		public GuiTrader(T trader)
		{
			this.Trader = trader;
		}

		/// <summary>
		///   .          1 .
		/// </summary>
		/// <param name="trader">,     <see cref="GuiTrader{T}"/>.</param>
		/// <param name="dispatcher">     GUI .</param>
		public GuiTrader(T trader, Dispatcher dispatcher)
			: base(dispatcher)
		{
			this.Trader = trader;
		}

		/// <summary>
		///   .
		/// </summary>
		/// <param name="trader">,     <see cref="GuiTrader{T}"/>.</param>
		/// <param name="dispatcher">     GUI .</param>
		/// <param name="interval">       .</param>
		public GuiTrader(T trader, Dispatcher dispatcher, TimeSpan interval)
			: base(dispatcher, interval)
		{
			this.Trader = trader;
		}

		private T _trader;

		/// <summary>
		///   .
		/// </summary>
		public T Trader
		{
			get { return _trader; }
			private set
			{
				if (value.IsNull())
					throw new ArgumentNullException();

				_trader = value;

				this.Trader.NewPortfolios += NewPortfoliosHandler;
				this.Trader.PortfoliosChanged += PortfoliosChangedHandler;
				this.Trader.NewPositions += NewPositionsHandler;
				this.Trader.PositionsChanged += PositionsChangedHandler;
				this.Trader.NewSecurities += NewSecuritiesHandler;
				this.Trader.SecuritiesChanged += SecuritiesChangedHandler;
				this.Trader.NewTrades += NewTradesHandler;
				this.Trader.NewMyTrades += NewMyTradesHandler;
				this.Trader.NewOrders += NewOrdersHandler;
				this.Trader.OrdersChanged += OrdersChangedHandler;
				this.Trader.OrdersFailed += OrdersFailedHandler;
				this.Trader.NewStopOrders += NewStopOrdersHandler;
				this.Trader.StopOrdersChanged += StopOrdersChangedHandler;
				this.Trader.StopOrdersFailed += StopOrdersFailedHandler;
				this.Trader.QuotesChanged += QuotesChangedHandler;
				this.Trader.Connected += ConnectedHandler;
				this.Trader.Disconnected += DisconnectedHandler;
				this.Trader.ConnectionError += ConnectionErrorHandler;
				this.Trader.NewDataExported += NewDataExportedHandler;
				this.Trader.ProcessDataError += ProcessDataErrorHandler;
			}
		}

		#region NewPortfolios

		/// <summary>
		///    .
		/// </summary>
		public event Action<IEnumerable<Portfolio>> NewPortfolios;

		private void NewPortfoliosHandler(IEnumerable<Portfolio> portfolios)
		{
			base.EventDispatcher.Add(() => NewPortfolios.SafeInvoke(portfolios));
		}

		#endregion

		#region PortfoliosChanged

		/// <summary>
		///    .
		/// </summary>
		public event Action<IEnumerable<Portfolio>> PortfoliosChanged;

		private void PortfoliosChangedHandler(IEnumerable<Portfolio> portfolios)
		{
			base.EventDispatcher.Add(() => PortfoliosChanged.SafeInvoke(portfolios));
		}

		#endregion

		#region NewPositions

		/// <summary>
		///    .
		/// </summary>
		public event Action<IEnumerable<Position>> NewPositions;

		private void NewPositionsHandler(IEnumerable<Position> positions)
		{
			base.EventDispatcher.Add(() => NewPositions.SafeInvoke(positions));
		}

		#endregion

		#region PositionsChanged

		/// <summary>
		///    .
		/// </summary>
		public event Action<IEnumerable<Position>> PositionsChanged;

		private void PositionsChangedHandler(IEnumerable<Position> positions)
		{
			base.EventDispatcher.Add(() => PositionsChanged.SafeInvoke(positions));
		}

		#endregion

		#region NewSecurities

		/// <summary>
		///    .
		/// </summary>
		public event Action<IEnumerable<Security>> NewSecurities;

		private void NewSecuritiesHandler(IEnumerable<Security> securities)
		{
			base.EventDispatcher.Add(() => NewSecurities.SafeInvoke(securities));
		}

		#endregion

		#region SecuritiesChanged

		/// <summary>
		///    .
		/// </summary>
		public event Action<IEnumerable<Security>> SecuritiesChanged;

		private void SecuritiesChangedHandler(IEnumerable<Security> securities)
		{
			base.EventDispatcher.Add(() => SecuritiesChanged.SafeInvoke(securities));
		}

		#endregion

		#region NewTrades

		/// <summary>
		///     .
		/// </summary>
		public event Action<IEnumerable<Trade>> NewTrades;

		private void NewTradesHandler(IEnumerable<Trade> trades)
		{
			base.EventDispatcher.Add(() => NewTrades.SafeInvoke(trades));
		}

		#endregion

		#region NewMyTrades

		/// <summary>
		///     .
		/// </summary>
		public event Action<IEnumerable<MyTrade>> NewMyTrades;

		private void NewMyTradesHandler(IEnumerable<MyTrade> trades)
		{
			base.EventDispatcher.Add(() => NewMyTrades.SafeInvoke(trades));
		}

		#endregion

		#region NewOrders

		/// <summary>
		///    .
		/// </summary>
		public event Action<IEnumerable<Order>> NewOrders;

		private void NewOrdersHandler(IEnumerable<Order> orders)
		{
			base.EventDispatcher.Add(() => NewOrders.SafeInvoke(orders));
		}

		#endregion

		#region OrdersChanged

		/// <summary>
		///     (, ).
		/// </summary>
		public event Action<IEnumerable<Order>> OrdersChanged;

		private void OrdersChangedHandler(IEnumerable<Order> orders)
		{
			base.EventDispatcher.Add(() => OrdersChanged.SafeInvoke(orders));
		}

		#endregion

		#region OrdersFailed

		/// <summary>
		///   ,   .
		/// </summary>
		public event Action<IEnumerable<OrderFail>> OrdersFailed;

		private void OrdersFailedHandler(IEnumerable<OrderFail> fails)
		{
			base.EventDispatcher.Add(() => OrdersFailed.SafeInvoke(fails));
		}

		#endregion

		#region NewStopOrders

		/// <summary>
		///    -.
		/// </summary>
		public event Action<IEnumerable<Order>> NewStopOrders;

		private void NewStopOrdersHandler(IEnumerable<Order> orders)
		{
			base.EventDispatcher.Add(() => NewStopOrders.SafeInvoke(orders));
		}

		#endregion

		#region StopOrdersChanged

		/// <summary>
		///    -.
		/// </summary>
		public event Action<IEnumerable<Order>> StopOrdersChanged;

		private void StopOrdersChangedHandler(IEnumerable<Order> orders)
		{
			base.EventDispatcher.Add(() => StopOrdersChanged.SafeInvoke(orders));
		}

		#endregion

		#region StopOrdersFailed

		/// <summary>
		///   ,   -.
		/// </summary>
		public event Action<IEnumerable<OrderFail>> StopOrdersFailed;

		private void StopOrdersFailedHandler(IEnumerable<OrderFail> fails)
		{
			base.EventDispatcher.Add(() => StopOrdersFailed.SafeInvoke(fails));
		}

		#endregion

		#region QuotesChanged

		/// <summary>
		///     .
		/// </summary>
		public event Action<IEnumerable<MarketDepth>> QuotesChanged;

		private void QuotesChangedHandler(IEnumerable<MarketDepth> marketDepths)
		{
			base.EventDispatcher.Add(() => QuotesChanged.SafeInvoke(marketDepths));
		}

		#endregion

		#region Connected

		/// <summary>
		///   .
		/// </summary>
		public event Action Connected;

		private void ConnectedHandler()
		{
			base.EventDispatcher.Add(() => Connected.SafeInvoke());
		}

		#endregion

		#region Disconnected

		/// <summary>
		///   .
		/// </summary>
		public event Action Disconnected;

		private void DisconnectedHandler()
		{
			base.EventDispatcher.Add(() => Disconnected.SafeInvoke());
		}

		#endregion

		#region ConnectionError

		/// <summary>
		///    (,   ).
		/// </summary>
		public event Action<Exception> ConnectionError;

		private void ConnectionErrorHandler(Exception exception)
		{
			base.EventDispatcher.Add(() => ConnectionError.SafeInvoke(exception));
		}

		#endregion

		#region NewDataExported

		/// <summary>
		/// ,     .
		/// </summary>
		public event Action NewDataExported;

		private void NewDataExportedHandler()
		{
			base.EventDispatcher.Add(() => NewDataExported.SafeInvoke());
		}

		#endregion

		#region ProcessDataError

		/// <summary>
		/// ,           .
		/// </summary>
		public event Action<Exception> ProcessDataError;

		private void ProcessDataErrorHandler(Exception exception)
		{
			base.EventDispatcher.Add(() => ProcessDataError.SafeInvoke(exception));
		}

		#endregion

		/// <summary>
		///    .
		///    ,    <see cref="ITrader.NewSecurities" />.     .
		/// </summary>
		public IEnumerable<Security> Securities
		{
			get { return this.Trader.Securities; }
		}

		/// <summary>
		///  .  ,       ,   .
		///  <see cref="ITrader.MarketTime"/>       .
		/// </summary>
		public TimeSpan MarketTimeOffset
		{
			get { return this.Trader.MarketTimeOffset; }
			set { this.Trader.MarketTimeOffset = value; }
		}

		/// <summary>
		///   .
		/// </summary>
		public DateTime MarketTime
		{
			get { return this.Trader.MarketTime; }
		}

		/// <summary>
		///   .
		/// </summary>
		public IEnumerable<Order> Orders
		{
			get { return this.Trader.Orders; }
		}

		/// <summary>
		///   -.
		/// </summary>
		public IEnumerable<Order> StopOrders
		{
			get { return this.Trader.StopOrders; }
		}

		/// <summary>
		///     .
		/// </summary>
		public IEnumerable<OrderFail> OrderFails
		{
			get { return this.Trader.OrderFails; }
		}

		/// <summary>
		///   .
		/// </summary>
		public IEnumerable<Trade> Trades
		{
			get { return this.Trader.Trades; }
		}

		/// <summary>
		///    .
		/// </summary>
		public IEnumerable<MyTrade> MyTrades
		{
			get { return this.Trader.MyTrades; }
		}

		/// <summary>
		///   .
		/// </summary>
		public IEnumerable<Portfolio> Portfolios
		{
			get { return this.Trader.Portfolios; }
		}

		/// <summary>
		///   .
		/// </summary>
		public IEnumerable<Position> Positions
		{
			get { return this.Trader.Positions; }
		}

		/// <summary>
		///  .
		/// </summary>
		public bool IsConnected
		{
			get { return this.Trader.IsConnected; }
		}

		/// <summary>
		///   .     <see cref="ITrader.StartExport"/>.
		/// </summary>
		public bool IsExportRunning
		{
			get { return this.Trader.IsExportRunning; }
		}

		/// <summary>
		///    .
		/// </summary>
		public void Connect()
		{
			this.Trader.Connect();
		}

		/// <summary>
		///    .
		/// </summary>
		public void Disconnect()
		{
			this.Trader.Disconnect();
		}

		/// <summary>
		///    .
		/// </summary>
		public void Reconnect()
		{
			this.Trader.Reconnect();
		}

		/// <summary>
		///      .
		/// </summary>
		/// <param name="portfolio">,     .</param>
		/// <param name="security">,     .</param>
		/// <returns>.    ,    null.</returns>
		public Position GetPosition(Portfolio portfolio, Security security)
		{
			return this.Trader.GetPosition(portfolio, security);
		}

		/// <summary>
		///   .
		/// </summary>
		/// <param name="security">,     .</param>
		/// <returns> .</returns>
		public MarketDepth GetMarketDepth(Security security)
		{
			return this.Trader.GetMarketDepth(security);
		}

		/// <summary>
		///      .
		/// </summary>
		/// <param name="portfolio">,       .</param>
		public void RegisterPortfolio(Portfolio portfolio)
		{
			this.Trader.RegisterPortfolio(portfolio);
		}

		/// <summary>
		///      .
		/// </summary>
		/// <param name="portfolio">,       .</param>
		public void UnRegisterPortfolio(Portfolio portfolio)
		{
			this.Trader.UnRegisterPortfolio(portfolio);
		}

		/// <summary>
		///     (, <see cref="Security.LastTrade"/>  <see cref="Security.BestBid"/>)  .
		/// </summary>
		/// <param name="security">,       .</param>
		public void RegisterSecurity(Security security)
		{
			this.Trader.RegisterSecurity(security);
		}

		/// <summary>
		///    .
		/// </summary>
		/// <param name="security">,       .</param>
		public void UnRegisterSecurity(Security security)
		{
			this.Trader.UnRegisterSecurity(security);
		}

		/// <summary>
		///    .
		/// </summary>
		/// <param name="order">,    .</param>
		public void RegisterOrder(Order order)
		{
			this.Trader.RegisterOrder(order);
		}

		/// <summary>
		///    .
		/// </summary>
		/// <param name="oldOrder">,   .</param>
		/// <param name="newOrder"> ,   .</param>
		public void ReRegisterOrder(Order oldOrder, Order newOrder)
		{
			this.Trader.ReRegisterOrder(oldOrder, newOrder);
		}

		/// <summary>
		///    .
		/// </summary>
		/// <param name="oldOrder">,         .</param>
		/// <param name="price">  .</param>
		/// <param name="volume">  .</param>
		/// <returns> .</returns>
		public Order ReRegisterOrder(Order oldOrder, double price, int volume)
		{
			return this.Trader.ReRegisterOrder(oldOrder, price, volume);
		}

		/// <summary>
		///    .
		/// </summary>
		/// <param name="order">,   .</param>
		public void CancelOrder(Order order)
		{
			this.Trader.CancelOrder(order);
		}

		/// <summary>
		///       .
		/// </summary>
		/// <param name="isStopOrder">True,     -, false -     null -   .</param>
		/// <param name="portfolio">.    null,        .</param>
		/// <param name="direction"> .    null,        .</param>
		/// <param name="classCode"> .    ,        .</param>
		/// <param name="security">.    null,        .</param>
		public void CancelOrders(bool? isStopOrder = null, Portfolio portfolio = null, OrderDirections? direction = null, string classCode = null, Security security = null)
		{
			this.Trader.CancelOrders(isStopOrder, portfolio, direction, classCode, security);
		}

		/// <summary>
		///    ()  .
		///       <see cref="ITrader.GetMarketDepth"/>.
		/// </summary>
		/// <param name="security">,      .</param>
		public void RegisterQuotes(Security security)
		{
			this.Trader.RegisterQuotes(security);
		}

		/// <summary>
		///     .
		/// </summary>
		/// <param name="security">,      .</param>
		public void UnRegisterQuotes(Security security)
		{
			this.Trader.UnRegisterQuotes(security);
		}

		/// <summary>
		///    ( )  .     
		///  <see cref="ITrader.NewTrades"/>.
		/// </summary>
		/// <param name="security">,      .</param>
		public void RegisterTrades(Security security)
		{
			this.Trader.RegisterTrades(security);
		}

		/// <summary>
		///    ( )  .
		/// </summary>
		/// <param name="security">,      .</param>
		public void UnRegisterTrades(Security security)
		{
			this.Trader.UnRegisterTrades(security);
		}

		/// <summary>
		///         ( , ,   ..).
		/// </summary>
		public void StartExport()
		{
			this.Trader.StartExport();
		}

		/// <summary>
		///        ,   <see cref="ITrader.StartExport"/>.
		/// </summary>
		public void StopExport()
		{
			this.Trader.StopExport();
		}

		/// <summary>
		///         <see cref="ITrader.StopExport"/>  <see cref="ITrader.StartExport"/>.
		/// </summary>
		public void ReStartExport()
		{
			this.Trader.ReStartExport();
		}

		/// <summary>
		///   .
		/// </summary>
		protected override void DisposeManaged()
		{
			this.Trader.NewSecurities -= NewSecuritiesHandler;
			this.Trader.SecuritiesChanged -= SecuritiesChangedHandler;
			this.Trader.NewPortfolios -= NewPortfoliosHandler;
			this.Trader.PortfoliosChanged -= PortfoliosChangedHandler;
			this.Trader.NewPositions -= NewPositionsHandler;
			this.Trader.PositionsChanged -= PositionsChangedHandler;
			this.Trader.NewTrades -= NewTradesHandler;
			this.Trader.NewMyTrades -= NewMyTradesHandler;
			this.Trader.NewOrders -= NewOrdersHandler;
			this.Trader.OrdersChanged -= OrdersChangedHandler;
			this.Trader.OrdersFailed -= OrdersFailedHandler;
			this.Trader.NewStopOrders -= NewStopOrdersHandler;
			this.Trader.StopOrdersChanged -= StopOrdersChangedHandler;
			this.Trader.StopOrdersFailed -= StopOrdersFailedHandler;
			this.Trader.QuotesChanged -= QuotesChangedHandler;
			this.Trader.Connected -= ConnectedHandler;
			this.Trader.Disconnected -= DisconnectedHandler;
			this.Trader.ConnectionError -= ConnectionErrorHandler;
			this.Trader.NewDataExported -= NewDataExportedHandler;
			this.Trader.ProcessDataError -= ProcessDataErrorHandler;

			base.DisposeManaged();
		}
	}
}