﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.Generic;
	using System.Text;
	using System.Windows;
	using System.Windows.Input;

	using Ecng.Common;
	using Ecng.Trading.Algo.Strategies;
	using Ecng.Xaml;

	/// <summary>
	/// Графический компонент для отображения логов.
	/// </summary>
	public partial class LogControl
	{
		/// <summary>
		/// Создать <see cref="LogControl"/>.
		/// </summary>
		public LogControl()
		{
			this.Entries = new ThreadSafeObservableCollection<LogEntry>();
			this.TimeFormat = "HH:mm:ss";
			InitializeComponent();
		}

		/// <summary>
		/// Формат конвертирования времени в строку.
		/// </summary>
		public string TimeFormat { get; set; }

		/// <summary>
		/// Коллекция лог-записей.
		/// </summary>
		public IList<LogEntry> Entries { get; private set; }

		private void CopyCanExecute(object sender, CanExecuteRoutedEventArgs e)
		{
			e.CanExecute = true;
			e.Handled = true;
		}

		private void CopyExecuted(object sender, ExecutedRoutedEventArgs e)
		{
			var text = new StringBuilder();

			foreach (LogEntry item in LogEntryDetails.SelectedItems)
			{
				text.AppendFormat("{0} {1} {2}		{3}",
					item.Strategy != null ? item.Strategy.Name : string.Empty,
					item.Time, GetLogType(item.ErrorState), item.Message);
				text.AppendLine();
			}

			Clipboard.SetText(text.To<string>());
			e.Handled = true;
		}

		private static string GetLogType(StrategyErrorStates errorState)
		{
			switch (errorState)
			{
				case StrategyErrorStates.None:
					return string.Empty;
				case StrategyErrorStates.Warning:
					return "Внимание";
				case StrategyErrorStates.Error:
					return "Ошибка";
				default:
					throw new ArgumentOutOfRangeException("errorState");
			}
		}
	}
}