﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.Generic;
	using System.Linq;
	using System.Windows.Controls;

	using Ecng.Common;
	using Ecng.Xaml;
	using Ecng.Trading.BusinessEntities;

	/// <summary>
	/// Визуальный контрол-таблица, отображающая сделки (коллекцию объектов класса <see cref="MyTrade"/>).
	/// </summary>
	public partial class TradeGrid
	{
		/// <summary>
		/// Создать объект класса <see cref="TradeGrid"/>.
		/// </summary>
		public TradeGrid()
		{
			this.Trades = new ThreadSafeObservableCollection<MyTrade>();
			InitializeComponent();
		}

		/// <summary>
		/// Коллекция сделок, которые необходимо отобразить.
		/// </summary>
		public IList<MyTrade> Trades { get; private set; }

		/// <summary>
		/// Происходит при новом выделении сделки.
		/// </summary>
		public event EventHandler<EventArgs> TradeSelected;

		/// <summary>
		/// Выбранная сделка.
		/// </summary>
		public MyTrade SelectedTrade
		{
			get
			{
				return this.SelectedTrades.FirstOrDefault();
			}
		}

		/// <summary>
		/// Выбранные сделки.
		/// </summary>
		public IEnumerable<MyTrade> SelectedTrades
		{
			get
			{
				return _trades.SelectedItems.Cast<MyTrade>();
			}
		}

		private void _trades_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			this.TradeSelected.SafeInvoke(this);
		}
	}
}